      program dequantb
C   
C   This version of dequanta is written for Fortran 77.
C   It uses rwspc77
C    originally written by Yoram Apelblat in May 1997
C    modified by J. Bertie and Y. Apelblat in November 1997
C
C   The program reads spectra of the real, n, and imaginary, k,
C   refractive indices, and calculates, as instructed, the 
C   Molar absorption coefficient (Em), real and imaginary dielectric
C   constants (epsilon' and epsilon''), the real and imaginary molar
C   polarizability (alpha' and alpha'') and the 
C   imaginary molar polarizability * wavenumber spectra.
C   The molar volume must be supplied, in mL/mole.
C
C   The program checks that the n and k file have the same wavenumber
C   spacing and range.  It stops if the spacing is not the same.
C   If the ranges differ but the spacing is the same, the program
C   proceeds but only in the range of overlap.
C
C   Originally written by Yoram Apelblat in May 1997.
C   Modified by J. Bertie in November 1997.
C
      Parameter(NMAX=16384)
      real yk(NMAX),yn(NMAX),ytemp(NMAX),ys1(NMAX),ys2(NMAX)
      real xspk,xepk,xspn,xepn,xsp,xep,res
      integer xt,yt,nptk,nptn,npt,ier,units
      character*192 memo
      character*12 namek,namen, filein
      character*12 nameem,namedc,namedl,namerp,nameip,namena
      character*6 rootname 
      character*1 ans,ans2
c
      write(*,*)' Welcome to dequant' 
      write(*,*)'The maximum file size handled is ',NMAX
    5 write(*,*)' Input from Keyboard (K) or text file (F) ?' 
      read(*,'(a)')ans
      write(*,*)'  ',ans,' entered'
      IF((ans.NE.'k').and.(ans.NE.'K')) THEN 
        IF((ans.NE.'f').and.(ans.NE.'F'))THEN
	    WRITE(*,*) 'entered value is ans = ',ans
	    GOTO 5
        ELSE
	    CONTINUE
        ENDIF
    	ENDIF
    	
      if((ans.eq.'K').or.(ans.eq.'k')) then
C
C      INPUT FROM KEYBOARD SECTION
C
   10 write(*,*)' Enter input k spectrum file name with extension' 
      read(*,'(a)')namek
      write(*,*)' Enter input n spectrum file name with extension'
      read(*,'(a)')namen
      call rspc77(namek,yk,nptk,xspk,xepk,xt,yt,res,memo,ier)
      call rspc77(namen,yn,nptn,xspn,xepn,xt,yt,res,memo,ier)
      
C   CHECK THAT nptk or nptn don't exceed NMAX
      IF ((NPTK.GT.NMAX).OR.(NPTN.GT.NMAX))THEN
	    WRITE(*,*)' Files too large'
	    WRITE(*,*)'Files ',namek,' and ',namen,' ignored'
		Go To 105
	ENDIF
      
      write(*,*)' Enter the molar volume in mL/mol'
      read(*,*)vm
      write(*,*)' The molar volume is ',vm,' mL/mol'
c
c     check that the n and k files have the same range and spacing. 
c
      if((xspk.eq.xspn).and.(xepk.eq.xepn).and.(nptk.eq.nptn)) then
      xsp=xspk
      xep=xepk
      npt=nptk
      space=(xsp-xep)/(npt-1)
      write(*,*)' The n and k files have the same spacing and length.'
      else
      nflag=1
      spacek=(xspk-xepk)/(nptk-1)
      spacen=(xspn-xepn)/(nptn-1)
      write(*,*)'space k ',spacek,' space n ',spacen
      if(spacek.ne.spacen) then
      write(*,*)' The n and k files have different spacings.'
      WRITE(*,*)'Files ',namek,' and ',namen,' ignored.'
	Go To 105
      endif
      write(*,*)' The n and k files have different lengths.'
      write(*,*)' Output will be for the overlap region ,only.'
      xsp=xspk
      xep=xepk
      npt=nptk
      space=(xsp-xep)/(npt-1)
      if(xspn.lt.xspk) xsp=xspn
      if(xepn.gt.xepk) xep=xepn
      npt=(xsp-xep)/space+1
      endif
c
c     If yk and yn are not the same length, use temporary array ytemp 
c     while the lengths are made equal.
      if(nflag.eq.1) then
      indxk=nint((xspk-xsp)/space)
      do 15 i=1,npt
   15 ytemp(i)=yk(indxk+i)
      do 20 i=1,npt
   20 yk(i)=ytemp(i)
      indxn=nint((xspn-xsp)/space)
      do 25 i=1,npt
   25 ytemp(i)=yn(indxn+i)
      do 30 i=1,npt
   30 yn(i)=ytemp(i)
      endif
c     
      write(*,*)' Enter root name for output files.' 
      write(*,*)' Up to 6 characters '
      read(*,'(a)')rootname
c
c     checking the length of the root name and concatenating the 
c     appropriate output suffixes
c
   35 write(*,*)' Enter spectra to calculate'
      write(*,*)' 1 - Em spectrum' 
      write(*,*)' 2 - dielectric constants spectra'
      write(*,*)' 3 - molar polarizability spectra'
      read(*,*)n1
      call rootchk(rootname,nc)
C
      if(n1.eq.1) then
      nameem=rootname(1:nc)//'em.spc'
      write(*,*)' The output file will be:'
      write(*,*)' molar absorption coefficient (E molar) - ',nameem
      write(*,*)' The molar volume is ',vm,' mL/mol'
      write(*,*)' Desired units of Em ? :'
      write(*,*)' L/(mole-cm) (enter 1) or cm^2/mole (enter 2)'
      read(*,*)units
C
      elseif(n1.eq.2) then
      namedc=rootname(1:nc)//'dc.spc'
      namedl=rootname(1:nc)//'dl.spc'
      write(*,*)' The output files will be:'
      write(*,*)' real dielectric constant      - ',namedc
      write(*,*)' imaginary dielectric constant - ',namedl
C
      elseif(n1.eq.3) then
      namerp=rootname(1:nc)//'rp.spc'
      nameip=rootname(1:nc)//'ip.spc'
      namena=rootname(1:nc)//'na.spc'
      write(*,*)' The output files will be:'
      write(*,*)' real molar polarizability           - ',namerp
      write(*,*)' imaginary molar polarizability      - ',nameip
      write(*,*)' nu * imaginary molar polarizability - ',namena
      write(*,*)' The molar volume is ',vm,' mL/mol'
      else
      write(*,*)' n1:  Invalid entry'
      goto 35
      endif
c
c     calculate em, e', e'', a', a'' and nu*a''
c     the following equations are used:
c     Em = (4*pi*nu*k)/(ln 10 * c)        c = 1000 / Vm
c     e' = n^2-k^2 
c     e" = 2*n*k 
c     am' = (3*Vm/4*pi)*{(e'-1)(e'+2)+e"^2)}/[(e'+2)^2+e"^2]
c     am" = (9*Vm/4*pi)*e"/[(e'+2)^2+e"^2]
c     nuam" = nu*am"
c
      if(n1.eq.1) then
      do 40 j=1,npt
      xnu=xsp-space*(j-1)
   40 ys1(j)=0.00545705*xnu*yk(j)*vm
      If(units.EQ.1)write(*,*)' Em in L/mole cm-1'
      memo ='Decadic Molar Absorption Coefficient in L/(mole-cm)'
      if (units.EQ.2) then
          do 41 j=1,npt
   41     ys1(j)=ys1(j)*1000
          write(*,*)' Em in cm^2/mole'
          memo ='Decadic Molar Absorption Coefficient in cm^2/mole'
      endif
      call wspc77(nameem,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
C
      elseif(n1.eq.2) then
      do 45 j=1,npt
      ys1(j)=yn(j)*yn(j)-yk(j)*yk(j)
   45 ys2(j)=2.0*yn(j)*yk(j)
      memo ='(Real) dielectric constant'
      call wspc77(namedc,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
      memo ='Dielectric loss, i.e., imaginary dielectric constant'
      call wspc77(namedl,ys2,npt,xsp,xep,xt,yt,res,memo,ier)
C
      elseif(n1.eq.3) then
      do 50 j=1,npt
      ydc=yn(j)*yn(j)-yk(j)*yk(j)
      ydl=2.0*yn(j)*yk(j)
      zden=(ydc+2.0)**2+ydl**2
      ys1(j)=0.238732415*vm*((ydc-1.0)*(ydc+2.0)+ydl**2)/zden
   50 ys2(j)=0.716197244*vm*ydl/zden
      write(*,*)' real molar polarizability in cm^3/mole '
      memo ='RP, real molar polarizability, in cm^3/mole '
      call wspc77(namerp,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
      write(*,*)' imaginary molar polarizability in cm^3/mole'
      memo ='IP, Imaginary molar polarizability, in cm^3/mole '
      call wspc77(nameip,ys2,npt,xsp,xep,xt,yt,res,memo,ier)
      do 55 j=1,npt
      xnu=xsp-space*(j-1)
   55 ys1(j)=xnu*ys2(j)
      write(*,*)' nu*imaginary molar polarizability in cm^2/mole'
      memo ='nu*IP in cm^2/mole'
      call wspc77(namena,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
      endif
      write(*,*)' More calculations with same n and k? (Y/n)'
      read(*,'(a)')ans2
      if((ans2.eq.'Y').or.(ans2.eq.'y')) goto 35
  105  write(*,*)' More calculations with new n and k? (Y/n)'
      read(*,'(a)')ans2
      if((ans2.eq.'Y').or.(ans2.eq.'y')) goto 10
c      
C      END OF INPUT FROM KEYBOARD SECTION
C
C      START OF INPUT FROM TEXT FILE SECTION
C
      else
  110 write(*,*)' Enter input file name with extension'
      read(*,'(a)')filein
C   *** OPEN THE FILE
      CALL GETCHN(ICHAN,IER)
      OPEN(UNIT=ICHAN,FILE=filein,status='OLD',ERR=999)
      read(ICHAN,*)nrun
c
      do 60 m=1,nrun
      write(*,*)' Run ',m
      read(ICHAN,*)n1
      if(n1.eq.1) then
      read(ICHAN,*)namek,namen,rootname,vm,units
C     The n file is not used.  The name of any readable .SPC file can be given here.
      elseif(n1.eq.2) then
      read(ICHAN,*)namek,namen,rootname
      elseif(n1.eq.3) then
      read(ICHAN,*)namek,namen,rootname,vm
      endif
      call rspc77(namek,yk,nptk,xspk,xepk,xt,yt,res,memo,ier)
      call rspc77(namen,yn,nptn,xspn,xepn,xt,yt,res,memo,ier)
c
C   CHECK THAT nptk or nptn don't exceed NMAX
      IF ((NPTK.GT.NMAX).OR.(NPTN.GT.NMAX))THEN
	    WRITE(*,*)' Files too large'
	    WRITE(*,*)'Files ',namek,' and ',namen,' ignored'
		Go To 60
	  ENDIF
	  
c     check that the n and k files have the same range and spacing. 
c
      if((xspk.eq.xspn).and.(xepk.eq.xepn).and.(nptk.eq.nptn)) then
      xsp=xspk
      xep=xepk
      npt=nptk
      space=(xsp-xep)/(npt-1)
      else
      nflag=1
      spacek=(xspk-xepk)/(nptk-1)
      spacen=(xspn-xepn)/(nptn-1)
      if(spacek.ne.spacen) then
         write(*,*)' n and k files are of different spacings.'
         write(*,*)' Make sure files are of the same size and spacings'
         write(*,*)' Run ',m,' terminated'
         goto 60
      endif
      write(*,*)' Run ',m,' - n and k files have different lengths'
      write(*,*)' Output will be for the overlap region ,only.'
      xsp=xspk
      xep=xepk
      npt=nptk
      space=(xsp-xep)/(npt-1)
      if(xspn.lt.xspk) xsp=xspn
      if(xepn.gt.xepk) xep=xepn
      npt=(xsp-xep)/space+1
      endif
c
C     If yk and yn are not the same length, use temporary array ytemp 
c     while the lengths are made equal.
c
      if(nflag.eq.1) then
      indxk=nint((xspk-xsp)/space)
      do 65 i=1,npt
   65 ytemp(i)=yk(indxk+i)
      do 70 i=1,npt
   70 yk(i)=ytemp(i)
      indxn=nint((xspn-xsp)/space)
      do 75 i=1,npt
   75 ytemp(i)=yn(indxn+i)
      do 80 i=1,npt
   80 yn(i)=ytemp(i)
      endif
c
      call rootchk(rootname,nc)
      if(n1.eq.1) then
      nameem=rootname(1:nc)//'em.spc'
      elseif(n1.eq.2) then
      namedc=rootname(1:nc)//'dc.spc'
      namedl=rootname(1:nc)//'dl.spc'
      elseif(n1.eq.3) then
      namerp=rootname(1:nc)//'rp.spc'
      nameip=rootname(1:nc)//'ip.spc'
      namena=rootname(1:nc)//'na.spc'
      endif
c
c     calculate em, e', e'', a', a'' and nu*a''
c     the following equations are used:
c     Em = (4*pi*nu*k)/(ln 10 * c)      c = 1000 /vm
c     e' = n^2-k^2 
c     e" = 2*n*k 
c     am' = (3*Vm/4*pi)*{(e'-1)(e'+2)+e"^2)}/[(e'+2)^2+e"^2]
c     am" = (9*Vm/4*pi)*e"/[(e'+2)^2+e"^2]
c     nuam" = nu*am"
c
      if(n1.eq.1) then
      do 85 j=1,npt
      xnu=xsp-space*(j-1)
   85 ys1(j)=0.00545705*xnu*yk(j)*vm
      memo ='Decadic Molar Absorption Coefficient in L/(mole-cm)'
      if(units.EQ.2) then
          do 86 j=1,npt
   86     ys1(j)=ys1(j)*1000
          memo ='Decadic Molar Absorption Coefficient in cm^2/mole'
      endif
      call wspc77(nameem,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
C
      elseif(n1.eq.2) then
      do 90 j=1,npt
      ys1(j)=yn(j)*yn(j)-yk(j)*yk(j)
   90 ys2(j)=2.0*yn(j)*yk(j)
      memo ='(Real) dielectric constant'
      call wspc77(namedc,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
      memo ='Dielectric loss, i.e., imaginary dielectric constant'
      call wspc77(namedl,ys2,npt,xsp,xep,xt,yt,res,memo,ier)
C
      elseif(n1.eq.3) then
      do 95 j=1,npt
      ydc=yn(j)*yn(j)-yk(j)*yk(j)
      ydl=2.0*yn(j)*yk(j)
      zden=(ydc+2.0)**2+ydl**2
      ys1(j)=0.238732415*vm*((ydc-1.0)*(ydc+2.0)+ydl**2)/zden
   95 ys2(j)=0.716197244*vm*ydl/zden
      memo ='RP, real molar polarizability, in cm^3/mole '
      call wspc77(namerp,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
      memo ='IP, Imaginary molar polarizability, in cm^3/mole '
      call wspc77(nameip,ys2,npt,xsp,xep,xt,yt,res,memo,ier)
      do 100 j=1,npt
      xnu=xsp-space*(j-1)
  100 ys1(j)=xnu*ys2(j)
      memo ='nu*IP in cm^2/mole'
      call wspc77(namena,ys1,npt,xsp,xep,xt,yt,res,memo,ier)
      endif
   60 continue
      endif
C
C      End of input from Text file section
C
      stop
  999 Write(*,935)filein
      GO TO 110
  935 FORMAT(' File',A12,' can not be opened; enter new file name')
      end

      subroutine rootchk(check,nc)
c
c     subroutine checks the length of the root name
c
      character*6 check
      if(check(2:2).eq.' ') then
      nc=1
      elseif(check(3:3).eq.' ') then
      nc=2
      elseif(check(4:4).eq.' ') then
      nc=3
      elseif(check(5:5).eq.' ') then
      nc=4
      elseif(check(6:6).eq.' ') then
      nc=5
      else
      nc=6
      endif
      return
      end
